// pulser - airborne galloping ghost decoder/driver
// ceptimus.  Started May 2018  Modifed August 2018 to auto-detect if elevator and aux channels are connected and work with/without them automatically
// Original was for ATmega328. This version for ATtiny85 February 2019.  Burn bootloader for internal 16Mhz no crystal.
// rudder input from receiver on Pin5, elevator on Pin6 - these are actual DIL package pin numbers, Pin5 is PB0, Pin6 is PB1
// this 'potVersion', suggested by Spike, uses a pot on Pin7 (analog input) to control the drive power, rather than an additional receiver channel
// all inputs have range 1ms = 0% to 2ms = 100%  Note: Taranis + Multimodule + Spektrum receiver only about 1120 - 1920 us range.
// rudder input 0% = 100% left, 100% = 100% right.  Typical usable range from 30% to 70%
// elevator input 0% = full down, fastest pulsing rate of 12 cycles per second.  100% = full up, slowest pulsing rate of 2 cycles per second.
// analogu input from pot sets percentage of time motor H-bridge is powered (total of left and right phases) from 0% to 100%.

// leave #define in place for L9110(S) and other H-bridges that have active low inputs.  Comment out for MX1508, L298, TB6612FNG and other H-bridges with active high inputs 
// #define L9110

// default value to use for 'ELE' (pulse rate) input when input is not connected. 1300 gives a fairly fast pulse rate
#define DEFAULT_ELE 1300

#ifdef L9110
  #define OFF HIGH
  #define ON LOW
#else
  #define OFF LOW
  #define ON HIGH
#endif

#include "receiver.h"

// actual DIL package pin numbers for output drive: right is Pin2 (PB3) left is Pin3 (PB4)
#define RIGHT 3
#define LEFT 4

uint32_t remaining, eventTime;

void setup(void) {
  pinMode(RIGHT, OUTPUT);
  digitalWrite(RIGHT, OFF); // set both H-bridge pins off for idle (coast) state 
  pinMode(LEFT, OUTPUT);
  digitalWrite(LEFT, OFF);
   
  receiver::start(); // start receiving 2 channels on pins D6 and D5
  
  do {
    delay(10);
  } while (!receiver::active(0)); // wait for a valid receiver signal on the rudder channel
  
  eventTime = micros();
}


void loop(void) {
  // calculations of next set of event times all happen in the final 'coast' stage set up by the previous loop
  uint16_t cycle = receiver::active(1) ? receiver::channel(1) : DEFAULT_ELE; // elevator input: nominally 1000 - 2000 = 0 - 100%
  if (cycle < 1500) { // tweak input range to alter range 1000:1500:2000 to about 1100:1325:1800
    cycle = 1325 - ((1500 - cycle) >> 1);
  } else {
    cycle -= 175;
  }
  cycle = cycle > 1000 ? cycle > 2000 ? 1200 : cycle - 800 : 200; // shift and clip to 200 - 1200 range to give 6:1 ratio between fast and slow
  uint32_t cycleMicros = 417UL * cycle; // cycleMicros is length of one complete pulse cycle in microseconds.  Range 83400 to 500400 - about 12 Hz to 2 Hz

  uint16_t power = !receiver::active(0) ? 1125 : analogRead(1) + 1125; // auxilliary input: nominally 1125 - 1875 = 0 - 100% power but default to zero power if rudder signal not active
  power = power > 1125 ? power > 1875 ? 750 : power - 1125 : 0; // shift and clip to range 0 - 750
  power /= 3; // divide by three to give 0 - 250
  uint32_t poweredMicros = (cycleMicros >> 8) * power; // divide total cycle time by 256 and multiply by power to give a (total of left and right) powered duration of 0 to 250/256 of the total cycle time.

  uint16_t right = receiver::channel(0);  // amount of 'right': 1000 = 0% right to 2000 = 100% right
  right = right > 1000 ? right > 2000 ? 1000 : right - 1000 : 0; // shift and clip to range 0 - 1000
  right = right >> 1; // divide by two to give range 0 - 500 for 0% to 100% right
  uint16_t left = 500 - right; // left + right always totals 500

  uint32_t powerDelta = poweredMicros >> 9; // powerDelta is 512 times shorter than poweredMicros
  uint32_t rightMicros = powerDelta * right; // 'right' output duration
  uint32_t leftMicros = powerDelta * left; // 'left' output duration
  
  uint32_t coastInterval = (cycleMicros - poweredMicros) >> 1; // two equal-sized coast intervals pad out the periods from (1) switching off 'right' to switching on 'left' and (2) switching off 'left' till switching on 'right' in the next cycle

  // output the LEFT and RIGHT phases
  do { remaining = eventTime - micros(); } while (remaining < 0x80000000UL); // wait till micros() > eventTime, allowing for the wrap that occurs when micros() counts past 0xFFFFFFFF every 71 minutes 35 seconds (if flight should ever last that long!)
  digitalWrite(RIGHT, ON);
  eventTime += rightMicros;
  do { remaining = eventTime - micros(); } while (remaining < 0x80000000UL);
  digitalWrite(RIGHT, OFF);
  eventTime += coastInterval;
  do { remaining = eventTime - micros(); } while (remaining < 0x80000000UL);
  digitalWrite(LEFT, ON);
  eventTime += leftMicros;
  do { remaining = eventTime - micros(); } while (remaining < 0x80000000UL);
  digitalWrite(LEFT, OFF);
  eventTime += coastInterval;
}

